# This module will contain helper functions/classes for prescriptions
from taminsdk.resources.prescription import prescriptions_endpoint

try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin


# Make API calls
# interface/epresc/SendEpresc/<specific_endpoint>
def make_get_request(session, endpoint, params_data=None):
    url = f"{session.url}{prescriptions_endpoint}{endpoint}"
    return session.session.get(url, params=params_data, verify=True)


def make_post_request(session, endpoint, json_data):
    url = f"{session.url}{prescriptions_endpoint}{endpoint}"
    return session.session.post(url, json=json_data, verify=True)


def make_put_request(session, endpoint, headers=None, params_data=None,
                     form_data=None, json_data=None):
    url = f"{session.url}{prescriptions_endpoint}{endpoint}"
    return session.session.put(url, headers=headers, params=params_data,
                               data=form_data, json=json_data, verify=True)



class Service:
    """
    Create a Prescription object from the JSON data
    retrieved from the API
    """

    def __init__(self, services_data):
        for k in iter(services_data):
            setattr(self, k, services_data[k])

    def __iter__(self):
        ''' Returns the Iterator object '''
        return ServiceIterator(self)


class ServiceIterator:
    ''' Iterator class '''

    def __init__(self, service):
        # Service object reference
        self._service = service
        # member variable to keep track of current index
        self._index = 0

    def __next__(self):
        ''''Returns the next value from service object's lists '''
        if self._index < (len(self._service)):
            result = self._service[self._index]
            self._index += 1
            return result
            # End of Iteration
        raise StopIteration



from urllib.parse import urljoin

import requests

from taminsdk.exceptions import UserLoginException


class Session():
    """
    This class will manage a HTTP session to the tamin.ir API
    """

    def __init__(self, oauth_token=None, url=None, username=None, password=None, need_token=True):
        self.session = requests.Session()
        if url is not None:
            self.url = url
        else:

            self.url = 'https://ep-test.tamin.ir/api/'
        print(self.url)
        if need_token:
            if not oauth_token:
                oauth_token = self._login(username, password)

            # Set default headers
            default_headers = {
                'Authorization': f'Bearer {oauth_token}',
            }
            self.session.headers.update(default_headers)

    def _login(self, user, password):
        json_data = {
            'client_id': user,
            'secret': password
        }
        url = urljoin(self.url, f'ws/api/auth/login')
        response = self.session.post(url, json=json_data, verify=True)
        json_data = response.json()
        if response.status_code == 200:
            return json_data['data']
        else:
            raise UserLoginException(
                data=json_data['data'],
                status=json_data['status'],
                family=json_data['family'],
                reason=json_data['reason'])



"""
This module contains functions for prescription operations
"""
import datetime
import logging

from taminsdk.resources.service import (
    make_get_request, handle_response,
)
from taminsdk.resources.service.exceptions import (
    ServiceException,
)
from taminsdk.resources.service.types import Service

try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin

log = logging.getLogger(__name__)


def get_all_services(
        session,
        params_data=None
):
    """
    Get all Services
    """

    # get /api/ws-services
    start_time = datetime.datetime.now()
    response = make_get_request(
        session=session,
        endpoint='ws-services',
        params_data=params_data
    )
    duration = datetime.datetime.now() - start_time
    log.info('Response[%d]: %s, Duration: %s.%ss.' % (
        response.status_code, response.reason, duration.seconds, duration.microseconds))
    return handle_response(response, response.content.decode('utf-8'))


def get_prescription_type(
        session,
        params_data=None
):
    """
    Get prescription_type
    """

    # get /api/get_prescription_type
    start_time = datetime.datetime.now()
    response = make_get_request(
        session=session,
        endpoint='ws-prescription-type',
        params_data=params_data
    )
    duration = datetime.datetime.now() - start_time
    log.info('Response[%d]: %s, Duration: %s.%ss.' % (
        response.status_code, response.reason, duration.seconds, duration.microseconds))
    return handle_response(response, response.content.decode('utf-8'))


def get_paraclinic_taref(
        session,
        params_data=None
):
    """
    Get Paraclinic Taref
    """

    # get /api/ws-par-taref
    start_time = datetime.datetime.now()
    response = make_get_request(
        session=session,
        endpoint='ws-par-taref',
        params_data=params_data
    )
    duration = datetime.datetime.now() - start_time
    log.info('Response[%d]: %s, Duration: %s.%ss.' % (
        response.status_code, response.reason, duration.seconds, duration.microseconds))
    return handle_response(response, response.content.decode('utf-8'))


def get_drug_amount(
        session,
        params_data=None
):
    """
    Get Drug Amount
    """

    # get /api/get_drug_amount
    start_time = datetime.datetime.now()
    response = make_get_request(
        session=session,
        endpoint='ws-drug-amount',
        params_data=params_data
    )
    duration = datetime.datetime.now() - start_time
    log.info('Response[%d]: %s, Duration: %s.%ss.' % (
        response.status_code, response.reason, duration.seconds, duration.microseconds))
    return handle_response(response, response.content.decode('utf-8'))


def get_drug_instruction(
        session,
        params_data=None
):
    """
    Get Drug Amount
    """

    # get /api/ws-drug-instruction
    start_time = datetime.datetime.now()
    response = make_get_request(
        session=session,
        endpoint='ws-drug-instruction',
        params_data=params_data
    )
    duration = datetime.datetime.now() - start_time
    log.info('Response[%d]: %s, Duration: %s.%ss.' % (
        response.status_code, response.reason, duration.seconds, duration.microseconds))
    return handle_response(response, response.content.decode('utf-8'))



# This module will contain helper functions/classes for services
import json

from taminsdk import exceptions
from taminsdk.resources.service import services_endpoint

try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin


# Make API calls
# ws-services?serviceType=17
def make_get_request(session, endpoint, params_data=None):
    url = f"{session.url}{services_endpoint}{endpoint}"
    return session.session.get(url, params=params_data, verify=True)


def make_post_request(session, endpoint, json_data):
    url = f"{session.url}{services_endpoint}{endpoint}"
    return session.session.post(url, json=json_data, verify=True)


def make_put_request(session, endpoint, headers=None, params_data=None,
                     form_data=None, json_data=None):
    url = f"{session.url}{services_endpoint}{endpoint}"
    return session.session.put(url, headers=headers, params=params_data,
                               data=form_data, json=json_data, verify=True)


def handle_response(response, content):
    """Validate HTTP response
    """
    status = response.status_code
    if status in (301, 302, 303, 307):
        raise exceptions.Redirection(response, content)
    elif 200 <= status <= 299:
        return json.loads(content) if content else {}
    elif status == 400:
        raise exceptions.BadRequest(response, content)
    elif status == 401:
        raise exceptions.UnauthorizedAccess(response, content)
    elif status == 403:
        raise exceptions.ForbiddenAccess(response, content)
    elif status == 404:
        raise exceptions.ResourceNotFound(response, content)
    elif status == 405:
        raise exceptions.MethodNotAllowed(response, content)
    elif status == 409:
        raise exceptions.ResourceConflict(response, content)
    elif status == 410:
        raise exceptions.ResourceGone(response, content)
    elif status == 422:
        raise exceptions.ResourceInvalid(response, content)
    elif 401 <= status <= 499:
        raise exceptions.ClientError(response, content)
    elif 500 <= status <= 599:
        raise exceptions.ServerError(response, content)
    else:
        raise exceptions.ConnectionError(
            response, content, "Unknown response code: #{response.code}")



"""
This module contains functions for prescription operations
"""
import datetime
import logging

from taminsdk.resources.prescription import (
    make_get_request, make_post_request, )
from taminsdk.resources.prescription.types import (DocEprsc
                                                   )
from taminsdk.resources.service import handle_response

try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin

log = logging.getLogger(__name__)


def create_prescription(
        session,
        patient: str,
        mobile: str,
        doctor: DocEprsc,  # doc_eprc,
        doc_id: str,
        presc_date: datetime,
        doc_national_code: str,
        doc_mobile_no: str,
        presc_type: object,  # presc_type,
        comments: str,
        note_detail_eprscs: [object],  # note_detail_eprsc[],

):
    """
    Create a prescription
    """
    prescription_data = {

        'patient': patient,
        'mobile': mobile,
        'doctor': doctor,
        'doc_id': doc_id,
        'presc_date': presc_date,
        'doc_national_code': doc_national_code,
        'doc_mobile_no': doc_mobile_no,
        'presc_type': presc_type,
        'comments': comments,
        'note_detail_eprscs': note_detail_eprscs,
    }

    # POST /interface/epresc/SendEpresc
    start_time = datetime.datetime.now()
    response = make_post_request(session, '', json_data=prescription_data)
    duration = datetime.datetime.now() - start_time
    log.info('Response[%d]: %s, Duration: %s.%ss.' % (
        response.status_code, response.reason, duration.seconds, duration.microseconds))
    return handle_response(response, response.content.decode('utf-8'))


def get_prescription_detail(
        session,
        params_data=None
):
    """
    Get prescription_detail
    """

    # get /api/ws-services
    start_time = datetime.datetime.now()

    response = make_get_request(
        session=session,
        endpoint='',
        params_data=params_data
    )
    duration = datetime.datetime.now() - start_time
    log.info('Response[%d]: %s, Duration: %s.%ss.' % (
        response.status_code, response.reason, duration.seconds, duration.microseconds))
    return handle_response(response, response.content.decode('utf-8'))



class PrescriptionNotCreatedException(Exception):
    """
    Prescription could not be created
    """

    def __init__(self, message, error_code):
        super().__init__(message)
        self.error_code = error_code



#!/usr/bin/env python

import sys

import setuptools

dependencies = [
    'requests >= 2.0.0',
    'guzzle_sphinx_theme',
]

if sys.version_info < (3, 4):
    dependencies.append('enum34')

with open('PYPI_DESCRIPTION.rst') as file:
    long_description = file.read()

setuptools.setup(
    name='taminsdk',
    author='Mazafard',
    url='https://github.com/mazafard/tamin-sdk',
    author_email='mazafard@gmail.com',
    version='0.3.0',
    description='Unoffical Tamin SDK',
    long_description=long_description,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Topic :: Software Development",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        'Operating System :: OS Independent',
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    packages=setuptools.find_packages(),
    install_requires=dependencies,
)



from enum import IntEnum


class Prescription:
    """
    Create a Prescription object from the JSON data
    retrieved from the API
    """

    def __init__(self, prescription_data):
        for k in iter(prescription_data):
            setattr(self, k, prescription_data[k])


class DocType(object):

    def __init__(self, doc_type):
        self.doc_type = doc_type


class DocEprsc(DocType):

    def __init__(self):
        self.DOCTOR = '1'
        self.doc_type = super().__init__(doc_type=self.DOCTOR)


class PrescriptionType(IntEnum):
    """
    Type for Prescription
    """
    DRUG = 1
    PARACLINIC = 2
    VISIT = 3
    VISIT_SERVICE = 4
    SERVICE = 5
